<?php
	
	// En-tête
	$title = "Les boucles FOR et WHILE en PHP";
	include("../header.php");
	 
?>

<h2>Théorie</h2>

<div class="phrase">
	Une <b>boucle</b> ou <b>structure itérative</b> est un procédé permettant 
	de répéter un ou plusieurs traitements plusieurs fois.
	<br/><br/>
	La <b>boucle POUR</b> est commode pour exécuter un ou plusieurs traitements 
	un nombre de fois connu à l'avance. Au contraire, la <b>boucle TANT QUE</b> 
	est pratique pour exécuter un ou plusieurs traitements un nombre de fois 
	non connu à l'avance.
	<br/><br/>
	Pour créer une boucle POUR, on utilise le mot-clef <b>for</b>. En général, 
	on l'utilise avec un <b>compteur</b>, c'est-à-dire une variable qui augmente 
	ou diminue à chaque tour de boucle. On effectue les traitements à l'intérieur 
	de la boucle tant que le compteur n'a pas atteint un certain seuil.
	<br/><br/>
	Pour créer une boucle TANT QUE, on utilise le mot-clef <b>while</b>. On 
	utilise cette fois-ci une <b>condition d'arrêt</b>. On peut également 
	utiliser un compteur, auquel cas la condition d'arrêt portera sur la 
	valeur du compteur. Quoiqu'il en soit, on effectue les traitements à 
	l'intérieur de la boucle tant que la condition est vraie.
	<br/><br/>
	Les boucles sont de ce fait destinées à faciliter la mise en oeuvre de 
	traitements répétitifs, par exemple :
	<br/>
	- parcours et affichage des éléments d'un tableau ;
	<br/>
	- parcours et affichage des lignes d'une table d'une base de données ;
	<br/>
	- envoi d'un mail à une série d'utilisateurs ;
	<br/>
	- etc.
</div>

<h2>Exemples</h2>

<h3>Exemple 1</h3>

<div class="code">
	<code>
		// Affichage des nombres de 0 à 99:
		// $i=0 => signifie qu'on commence à boucler à partir de $i=0 ;
		// $i<100 => signifie qu'on boucle tant que $i est inférieur à 100 ;
		// $i++ => signifie qu'on $i augmente de 1 à chaque tour de boucle.
		// N.B.: une autre notation de $i++ est $i=$i+1.
		echo "Nombre de 0 à 99 : ";
		for($i=0; $i&lt;100; $i++){
			// Affichage de la valeur de la variable $i
			echo $i;
			// Affichage d'une chaîne pour séparer les nombres.
			if($i == 99){
				echo ".";
			}else{
				echo ", ";
			}
		}
	</code>
</div>

<div class="resultat">
	<?php 
		// Affichage des nombres de 0 à 99:
		// $i=0 => signifie qu'on commence à boucler à partir de $i=0 ;
		// $i<100 => signifie qu'on boucle tant que $i est inférieur à 100 ;
		// $i++ => signifie qu'on $i augmente de 1 à chaque tour de boucle.
		// N.B.: une autre notation de $i++ est $i=$i+1.
		echo "Nombre de 0 à 99 : ";
		for($i=0; $i<100; $i++){
			// Affichage de la valeur de la variable $i
			echo $i;
			// Affichage d'une chaîne pour séparer les nombres.
			if($i == 99){
				echo ".";
			}else{
				echo ", ";
			}
		}
	?>
</div>

<h3>Exemple 2</h3>

<div class="code">
	<code>
		// Affichage des nombres de 0 à 99 avec une boucle while.
		echo "Nombre de 0 à 99 : ";
		// Initialisation du compteur.
		$i = 0;
		while($i&lt;100){
			// Affichage de la valeur de la variable $i
			echo $i;
			// Affichage d'une chaîne pour séparer les nombres.
			if($i == 99){
				echo ".";
			}else{
				echo ", ";
			}
		}
	</code>
</div>

<div class="resultat">
	<?php 
		// Affichage des nombres de 0 à 99 avec une boucle while.
		echo "Nombre de 0 à 99 : ";
		// Initialisation du compteur.
		$i = 0;
		while($i<100){
			// Affichage de la valeur de la variable $i
			echo $i;
			// Affichage d'une chaîne pour séparer les nombres.
			if($i == 99){
				echo ".";
			}else{
				echo ", ";
			}
			// On augmente le compteur sinon on obtient une 
			// jolie boucle infinie ! On peut remplacer la ligne 
			// ci-dessous par $i = $i+1;
			$i++;
		}
	?>
</div>

<h3>Exemple 3</h3>

<div class="code">
	<code>
		// Création d'un tableau et récupération de sa taille.
		$fruits = array("Banane", "Fraise", "Kiwi", "Mangue", "Mûre");
		$nbFruits = count($fruits);
		
		// Parcours du tableau et affichage de la liste de fruits.
		echo "Fruits : ";
		for($i=0; $i&lt;$nbFruits-1; $i++){
			echo $fruits[$i] . ", ";
		}
		echo $fruits[$nbFruits-1] . ". &lt;br/>";
		
		// Pour information, une solution plus expéditive
		echo "Fruits : " . implode(", ", $fruits) . ". &lt;br/>";
		
		// Parcours du tableau et affichage de la liste de fruits 
		// sous forme de liste déroulante.
		echo "&lt;select>";
		for($i=0; $i&lt;$nbFruits; $i++){
			echo "&lt;option>" . $fruits[$i] . "&lt;/option>";
		}
		echo "&lt;/select>";
	</code>
</div>

<div class="resultat">
	<?php 
		// Création d'un tableau et récupération de sa taille.
		$fruits = array("Banane", "Fraise", "Kiwi", "Mangue", "Mûre");
		$nbFruits = count($fruits);
		
		// Parcours du tableau et affichage de la liste de fruits.
		echo "Fruits : ";
		for($i=0; $i<$nbFruits-1; $i++){
			echo $fruits[$i] . ", ";
		}
		echo $fruits[$nbFruits-1] . ". <br/>";
		
		// Pour information, une solution plus expéditive
		echo "Fruits : " . implode(", ", $fruits) . ". <br/>";
		
		// Parcours du tableau et affichage de la liste de fruits 
		// sous forme de liste déroulante.
		echo "<select>";
		for($i=0; $i<$nbFruits; $i++){
			echo "<option>" . $fruits[$i] . "</option>";
		}
		echo "</select>";
	?>
</div>

<?php
	
	// Pied de page
	include("../footer.php");
	
?>
