<?php
	
	// En-tête
	$title = "Les conditions en PHP";
	include("../header.php");
	 
?>

<h2>Les conditions</h2>

<h3>Théorie</h3>

<div class="phrase">
	En PHP comme dans tout langage de programmation, on peut effectuer 
	un traitement sous réserve qu'une condition soit vérifiée. On parle 
	de <b>structure conditionnelle</b> ou de branchement conditionnel.
	<br/><br/>
	Pour ce faire, on utilise le mot-clef <b>if</b> et, le cas échéant, 
	les mots-clefs <b>else if</b> et <b>else</b>.
	<br/><br/>
	Pour construire des conditions, on utilise des opérateurs logiques 
	(<b>et</b>, <b>ou</b>, <b>non</b>) et/ou des opérateurs de comparaison 
	(égal à, différent de, supérieur ou égal à, etc.). En PHP, ces 
	opérateurs sont : 
	<br/>
	- opérateurs logiques : <b>!</b> (non), <b>&&</b> (et), <b>||</b> (ou).
	<br/>
	- opérateurs de comparaison : <b>==</b> (égal à), <b>!=</b> (différent de), 
	<b>></b> (supérieur à), <b>>=</b> (supérieur ou égal à), etc. 
	<br/><br/>
	Egalement, on dispose d'un type de variables particulier, le type 
	<b>booléen</b>. Un booléen est une valeur ou variable qui est soit 
	vraie (<b>true</b>) soit fausse (<b>false</b>).
</div>

<h3>Exemple 1</h3>

<div class="code">
	<code>
		// Création de deux variables booléennes.
		$a = true;
		$b = false;
		
		// On affiche la valeur des variables
		if($a){
			echo "a => true &lt;br/>";
		}else{
			echo "a => false &lt;br/>";
		}
		if($b){
			echo "b => true &lt;br/>";
		}else{
			echo "b => false &lt;br/>";
		}
		
		// On affiche le résultat de l'opération logique $a ET $b.
		if($a && $b){
			echo "a ET b => true";
		}else{
			echo "a ET b => false";
		}
		echo "&lt;br/>";
		
		// On affiche le résultat de l'opération logique $a OU $b.
		if($a || $b){
			echo "a OU b => true";
		}else{
			echo "a OU b => false";
		}
		echo "&lt;br/>";
	</code>
</div>

<div class="resultat">
	<?php
			
		// Création de deux variables booléennes.
		$a = true;
		$b = false;
		
		// On affiche la valeur des variables
		if($a){
			echo "a => true <br/>";
		}else{
			echo "a => false <br/>";
		}
		if($b){
			echo "b => true <br/>";
		}else{
			echo "b => false <br/>";
		}
		
		// On affiche le résultat de l'opération logique $a ET $b.
		if($a && $b){
			echo "a ET b => true";
		}else{
			echo "a ET b => false";
		}
		echo "<br/>";
		
		// On affiche le résultat de l'opération logique $a OU $b.
		if($a || $b){
			echo "a OU b => true";
		}else{
			echo "a OU b => false";
		}
		echo "<br/>";
		
	?>
</div>

<h3>Exemple 2</h3>

<div class="code">
	<code>
		// Création de deux variables numériques 
		$nombre1 = 12;
		$nombre2 = 13;
		
		// On affiche la valeur des variables
		echo "nombre1 => " . $a . "&lt;br/>";
		echo "nombre2 => " . $a . "&lt;br/>";
		
		// Comparaison des deux nombres
		if($nombre1 > $nombre2){
			echo "comparaison => nombre1 est supérieur à nombre2";
		}else if($nombre1 == $nombre2){
			echo "comparaison => nombre1 est égal à nombre2";
		}else{
			echo "comparaison => nombre1 est inférieur à nombre2";
		}
	</code>
</div>

<div class="resultat">
	<?php
		
		// Création de deux variables numériques 
		$nombre1 = 12;
		$nombre2 = 13;
		
		// On affiche la valeur des variables
		echo "nombre1 => " . $a . "<br/>";
		echo "nombre2 => " . $a . "<br/>";
		
		// Comparaison des deux nombres
		if($nombre1 > $nombre2){
			echo "comparaison => nombre1 est supérieur à nombre2";
		}else if($nombre1 == $nombre2){
			echo "comparaison => nombre1 est égal à nombre2";
		}else{
			echo "comparaison => nombre1 est inférieur à nombre2";
		}
		
	?>
</div>

<h2>Les validations de données</h2>

<h3>Théorie</h3>

<div class="phrase">
	En PHP comme dans d'autres langages, il existe de multiples fonctions 
	permettant de faire des tests sur des données. En particulier, il est 
	souvent essentiel, afin d'obtenir un logiciel robuste et fiable, de 
	mettre en place des validations de données. <br/>
	<br/>
	Une <b>validation de données</b> permet de s'assurer qu'une ou plusieurs 
	valeur ont le type et/ou le format attendu. Par exemple, on peut vouloir 
	vérifier si oui ou non la valeur d'une variable est bien un nombre entier, 
	une chaîne représentant une adresse email, une chaîne représentant une 
	date au format jj/mm/aaaa, etc.
</div>

<h3>Exemple</h3>

<div class="code">
	<code>
		// Création de plusieurs variables
		$nombre = 5;
		$chaine = "Il était une fois...";
		$tableau1 = array(3, 6, 9, 12, 15);
		$tableau2 = array("a" => 0, "b" => 1, "c" => 2);
		
		// Teste si la variable $glups existe avec la fonction isset($maVariable).
		if(isset($glups)){
			echo "La variable 'glups' existe. &lt;br/>";
		}else{
			echo "La variable 'glups' n'existe pas. &lt;br/>";
		}
		
		// Teste si la variable $nombre existe.
		if(isset($nombre)){
			echo "La variable 'nombre' existe. &lt;br/>";
		}else{
			echo "La variable 'nombre' n'existe pas. &lt;br/>";
		}
		
		// Teste si l'élément $tableau2["b"] existe.
		if(isset($tableau2["b"])){
			echo "Le tableau associatif 'tableau2' a un élément à l'index 'b'. &lt;br/>";
		}
		
		// Teste si la valeur de la variable $chaine est un entier ou une chaîne
		if(is_int($chaine)){
			echo "La variable 'chaine' contient un entier. &lt;br/>";
		}else if(is_string($chaine)){
			echo "La variable 'chaine' contient une chaîne de caractères. &lt;br/>";
		}else{
			echo "La variable 'chaine' ne contient ni entier ni chaîne de caractères. &lt;br/>";
		}
		
		// Teste si la valeur de la variable $nombre est un entier compris 
		// entre 0 et 100.
		if(is_numeric($nombre)){
			if($nombre >= 0 && $nombre &lt;= 100){
				echo "La variable 'nombre' contient un entier compris entre 0 et 100. &lt;br/>";
			}else{
				echo "La variable 'nombre' contient un nombre inférieur à 0 ou supérieur à 100. &lt;br/>";
			}
		}
		
		// Teste si la longueur de la valeur de la variable $chaine est supérieure 
		// ou égale à 10.
		if(strlen($chaine) > 10){
			echo "La variable 'chaine' contient une chaîne d'au moins 10 caractères";
		}
		
	</code>
</div>

<div class="resultat">
	<?php
	
		// Création de plusieurs variables
		$nombre = 5;
		$chaine = "Il était une fois...";
		$tableau1 = array(3, 6, 9, 12, 15);
		$tableau2 = array("a" => 0, "b" => 1, "c" => 2);
		
		// Teste si la variable $glups existe avec la fonction isset($maVariable).
		if(isset($glups)){
			echo "La variable 'glups' existe. <br/>";
		}else{
			echo "La variable 'glups' n'existe pas. <br/>";
		}
		
		// Teste si la variable $nombre existe.
		if(isset($nombre)){
			echo "La variable 'nombre' existe. <br/>";
		}else{
			echo "La variable 'nombre' n'existe pas. <br/>";
		}
		
		// Teste si l'élément $tableau2["b"] existe.
		if(isset($tableau2["b"])){
			echo "Le tableau associatif 'tableau2' a un élément à l'index 'b'. <br/>";
		}
		
		// Teste si la valeur de la variable $chaine est un entier ou une chaîne
		if(is_int($chaine)){
			echo "La variable 'chaine' contient un entier. <br/>";
		}else if(is_string($chaine)){
			echo "La variable 'chaine' contient une chaîne de caractères. <br/>";
		}else{
			echo "La variable 'chaine' ne contient ni entier ni chaîne de caractères. <br/>";
		}
		
		// Teste si la valeur de la variable $nombre est un entier compris 
		// entre 0 et 100.
		if(is_numeric($nombre)){
			if($nombre >= 0 && $nombre <= 100){
				echo "La variable 'nombre' contient un entier compris entre 0 et 100. <br/>";
			}else{
				echo "La variable 'nombre' contient un nombre inférieur à 0 ou supérieur à 100. <br/>";
			}
		}
		
		// Teste si la longueur de la valeur de la variable $chaine est supérieure 
		// ou égale à 10.
		if(strlen($chaine) > 10){
			echo "La variable 'chaine' contient une chaîne d'au moins 10 caractères";
		}
		
	?>
</div>

<?php
	
	// Pied de page
	include("../footer.php");
	
?>
