<?php
	
	// En-tête
	$title = "Les paramètres GET, POST et SESSION en PHP";
	include("../header.php");
	 
?>

<h2>Le protocole HTTP</h2>

<div class="phrase">
	On rappelle qu'une application web (site internet, intranet, extranet ou 
	autre) est développée sur la base d'une <b>achitecture client-serveur</b>. 
	Le <b>logiciel client</b> est le <b>navigateur</b> (exemple : Chrome) et 
	le <b>logiciel serveur</b> est le <b>serveur web</b> (exemple : WampServer). 
	D'un point de vue matériel, lorsque nous développons des pages web à l'aide 
	de Notepad++ (ou autre) et de WampServer sur un ordinateur, notre ordinateur 
	joue tout à la fois le rôle du client et du serveur.
	<br/><br/>
	Pour communiquer entre eux, le logiciel client et le logiciel serveur 
	s'envoient des messages. Le client émet des <b>requêtes</b> et le serveur lui 
	retourne des <b>réponses</b>. Dans le cadre d'une application web, client et 
	serveur utilisent le <b>protocole HTTP</b>. On parle dès lors de <b>requêtes 
	HTTP</b> et de <b>réponses HTTP</b>.
	<br/><br/>
	Sur chrome*, vous pouvez visualiser les requêtes HTTP émises et les réponses 
	HTTP reçues en effectuant les opérations suivantes (voir capture ci-dessous) :
	<br/>
	- faire un clic droit sur la page web ;
	<br/>
	- cliquer sur "Inspecter" ;
	<br/>
	- cliquer sur l'onglet "Network". La liste des requêtes HTTP émises s'affiche 
	alors ;
	<br/>
	- cliquer sur l'URI de la requête à visualiser. Vous pouvez alors voir le 
	détail de la requête émise et de la réponse reçue (onglets "Headers" et 
	"Reponse").
	<br/><br/>
	<img class="capture" src="img/capture-chrome-inspecter.jpg" />
</div>

<h2>Les paramètres GET et POST</h2>

<h3>Théorie</h3>

<div class="phrase">
	Lorsqu'on envoie une requête HTTP, on peut envoyer des données au moyen 
	de <b>paramètres</b>. On parle de <b>paramètres GET</b> et de <b>paramètres 
	POST</b>. Ces paramètres ont une durée de vie limitée :
	<br/>
	- les paramètres sont envoyées via la requête HTTP ;
	<br/>
	- le serveur web reçoit la requête, dont les éventuels paramètres. On peut 
	les utiliser en PHP. Finalement, les paramètres ne sont pas conservés et 
	le serveur web retourne sa réponse HTTP.
	<br/><br/>
	Si on utilise l'inspecteur de Chrome, on peut visualiser les paramètres 
	transmis.
</div>

<h3>Paramètres GET</h3>

<div class="phrase">
	Les paramètres GET font partie de l'URL. Un exemple sera bien plus explicite. 
	Prenons l'URL suivante :
	<br/><br/>
	<a class="lien" href="php/exemples/parametres.php?a=5&b=7" rel="nofollow">
		http://localhost/tests/php/exemples/parametres.php?a=5&b=7
	</a>
	<br/><br/>
	Les paramètres sont les éléments situés après le caractère <b>?</b> : a=5&amp;b=7.
	<br/><br/>
	Un paramètre dispose d'un nom et d'une valeur. L'URL ci-dessus comporte deux paramètres, 
	les paramètres "a" et "b" qui valent respectivement "5" et "7".
	<br/><br/>
	Côté serveur, c'est-à-dire côté PHP, on peut récupérer et manipuler ces paramètres 
	grâce à la variable <b>$_GET</b>.
	<br/><br/>
	Vous pouvez par exemple cliquer sur le lien ci-dessus pour tester le bout de code 
	figurant ci-dessous.
</div>

<div class="code">
	<code>
		// On teste si les paramètres GET nommés "a" et "b" existent
		if(isset($_GET["a"], $_GET["b"])){
			// S'ils existent, on peut les utiliser
			$somme = $_GET["a"] + $_GET["b"];
			$produit = $_GET["a"] * $_GET["b"];
			// On affiche la valeur des paramètres puis leur somme et leur produit
			echo "a => " . $_GET["a"] . "&lt;br/>";
			echo "b => " . $_GET["b"] . "&lt;br/>";
			echo "a + b => " . $somme . "&lt;br/>";
			echo "a x b => " . $produit . "&lt;br/>";
		}else{
			echo "Les paramètres GET 'a' et 'b' ne sont pas définis.";
		}
	</code>
</div>

<div class="resultat">
	<?php 
		// On teste si les paramètres GET nommés "a" et "b" existent
		if(isset($_GET["a"], $_GET["b"])){
			// S'ils existent, on peut les utiliser
			$somme = $_GET["a"] + $_GET["b"];
			$produit = $_GET["a"] * $_GET["b"];
			// On affiche la valeur des paramètres puis leur somme et leur produit
			echo "a => " . $_GET["a"] . "<br/>";
			echo "b => " . $_GET["b"] . "<br/>";
			echo "a + b => " . $somme . "<br/>";
			echo "a x b => " . $produit . "<br/>";
		}else{
			echo "Les paramètres GET 'a' et 'b' ne sont pas définis.";
		}
	?>
</div>

<h3>Paramètres POST</h3>

<div class="phrase">
	Les paramètres POST font partie des en-têtes de la requête HTTP. Vous 
	pouvez les visualiser grâce à l'inspecteur Chrome mais ils ne figurent 
	pas dans l'URL contrairement aux paramètres POST.
	<br/><br/>
	Côté serveur, c'est-à-dire côté PHP, ces paramètres s'utilisent de la 
	même manière que les paramètres GET, à ceci près qu'on utilise la variable 
	<b>$_POST</b> et non plus la variable $_GET.
	<br/><br/>
	Les paramètres POST permettent typiquement de recevoir les données 
	provenant d'un formulaire quelconque :
	<br/>
	- on crée un formulaire grâce aux balises <b>&lt;form></b> et <b>&lt;/form></b> ;
	<br/>
	- on ajoute deux attributs à la balise &lt;form> : <b>method="POST"</b> et 
	<b>action="/uri/de/destination"</b>. Dans l'attribut "action", on précise 
	l'URI de la page web (PHP) qui traite le formulaire.
	<br/>
	- on ajoute ces champs par exemple au moyen de balises <b>&lt;input /></b>, 
	<b>&lt;textarea></b> et <b>&lt;/textarea></b>, <b>&lt;select></b> et <b>&lt;/select></b>. 
	On nomme tous les champs avec l'attribut <b>name</b>. Côté PHP, les paramètres 
	POST portent exactement le même nom que celui des champs de formulaires. Autrement 
	dit, à un champ de formulaire correspond normalement un paramètre POST ;
	<br/>
	- on ajoute finalement un bouton de soumission de formulaire soit grâce à 
	la balise <b>&lt;input type="submit" value="Envoyer" /></b> soit grâce aux 
	balises <b>&lt;button>Envoyer&lt;/button></b>.
	<br/><br/>
	On notera que les attributs <b>type</b>, <b>value</b> et <b>placeholder</b> des 
	champs de formulaire peuvent s'avérer fort utiles...
	<br/><br/>
	Comme toujours, un exemple vaudra mieux qu'un long discours.
</div>

<div class="code">
	<code>
		&lt;form method="POST" action="php/exemples/parametres.php">
			&lt;label>
				Prénom : &lt;input name="prenom" type="text" placeholder="[Votre prénom]" />
			&lt;/label>
			&lt;label>
				NOM : &lt;input name="nom" type="text" placeholder="[Votre prénom]" />
			&lt;/label>
			&lt;label>
				Appréciez-vous l'informatique ? &lt;input name="informatique" type="checkbox" checked />
			&lt;/label>
			&lt;label>
				Quel est votre matière préférée ? 
				&lt;select name="matiere">
					&lt;option value="maths">Mathématiques&lt;/option>
					&lt;option value="info" selected>Informatique&lt;/option>
					&lt;option value="eco">Economie-Droit&lt;/option>
					&lt;option value="mana">Management&lt;/option>
					&lt;option value="philo">Philosophie&lt;/option>
					&lt;option value="autre">Autre&lt;/option>
				&lt;/select>
			&lt;/label>
			&lt;button>Envoyer&lt;/button>
		&lt;/form>
		&lt;br/>
		&lt;?php
			// On teste si tous les paramètres postes on bien été reçus.
			if(isset(
				$_POST["prenom"], 
				$_POST["nom"], 
				$_POST["informatique"], 
				$_POST["matiere"]
			)){
				// On peut désormais utiliser les données du formulaire.
				// Affichons déjà les valeurs reçues.
				echo "Champ 'prenom' => " . $_POST["prenom"] . "&lt;br/>";
				echo "Champ 'nom' => " . $_POST["nom"] . "&lt;br/>";
				echo "Champ 'informatique' => " . $_POST["informatique"] . "&lt;br/>";
				echo "Champ 'matiere' => " . $_POST["matiere"] . "&lt;br/>";
				// Puis effectuons un traitement un peu plus compliqué.
				if($_POST["informatique"] == "on" && $_POST["matiere"] == "info"){
					echo "Vous êtes un winner !";
				}else if($_POST["informatique"] == "on" || $_POST["matiere"] == "info"){
					echo "Vous êtes un semi-winner...";
				}else{
					echo "Pfff! C'est bien l'informatique...";
				}
			}else{
				echo "Vous n'avez pas encore transmis le formulaire...";
			}
		?>
	</code>
</div>

<div class="resultat">
	<form method="POST" action="php/exemples/parametres.php">
		<label>
			Prénom : <input name="prenom" type="text" placeholder="[Votre prénom]" />
		</label>
		<label>
			NOM : <input name="nom" type="text" placeholder="[Votre prénom]" />
		</label>
		<label>
			Appréciez-vous l'informatique ? <input name="informatique" type="checkbox" checked />
		</label>
		<label>
			Quel est votre matière préférée ? 
			<select name="matiere">
				<option value="maths">Mathématiques</option>
				<option value="info" selected>Informatique</option>
				<option value="eco">Economie-Droit</option>
				<option value="mana">Management</option>
				<option value="philo">Philosophie</option>
				<option value="autre">Autre</option>
			</select>
		</label>
		<button>Envoyer</button>
	</form>
	<br/>
	<?php
		// On teste si tous les paramètres postes on bien été reçus.
		if(isset(
			$_POST["prenom"], 
			$_POST["nom"], 
			$_POST["informatique"], 
			$_POST["matiere"]
		)){
			// On peut désormais utiliser les données du formulaire.
			// Affichons déjà les valeurs reçues.
			echo "Champ 'prenom' => " . $_POST["prenom"] . "<br/>";
			echo "Champ 'nom' => " . $_POST["nom"] . "<br/>";
			echo "Champ 'informatique' => " . $_POST["informatique"] . "<br/>";
			echo "Champ 'matiere' => " . $_POST["matiere"] . "<br/>";
			// Puis effectuons un traitement un peu plus compliqué.
			if($_POST["informatique"] == "on" && $_POST["matiere"] == "info"){
				echo "Vous êtes un winner !";
			}else if($_POST["informatique"] == "on" || $_POST["matiere"] == "info"){
				echo "Vous êtes un semi-winner...";
			}else{
				echo "Pfff! C'est bien l'informatique...";
			}
		}else{
			echo "Vous n'avez pas encore transmis le formulaire...";
		}
	?>
</div>

<h2>Les paramètres de SESSION</h2>

<div class="phrase">
	Si les paramètres GET et POST ont une durée de vie très limitée, les 
	paramètres de session ont quant à eux une durée de vie moins limitée. 
	Leur durée de vie dépend de la configuration du serveur web. Ici, 
	grâce à quelques lignes de PHP, les paramètres de session ont 
	normalement une durée de vie de 30 minutes.
	<br/><br/>
	Les paramètres de session sont par exemple utilisés afin de conserver 
	des informations concernant l'utilisateur une fois que celui-ci s'est 
	authentifié au moyen d'un formulaire de connexion.
	<br/><br/>
	L'utilisation de ces paramètres est semblable à celle des paramètres 
	GET et POST à ceci près qu'on utilise la variable <b>$_SESSION</b>. 
	Elle aussi consiste en un tableau associatif.
	<br/><br/>
	Afin de pouvoir manipuler les paramètres de session, on doit appeler la 
	procédure <b>session_start()</b> avant tout code HTML, c'est-à-dire 
	au tout début de la ou des pages web.
	<br/><br/>
	Dans l'exemple ci-après, si vous complétez et transmettez le formulaire, 
	vous verrez apparaître la phrase "Votre prénom est ...". Rien d'étonnant, 
	certes. Mais si vous fermez le navigateur et rouvrez cette page, la phrase 
	est toujours là avec le prénom que vous avez saisi !
</div>

<div class="code">
	<code>
		&lt;form method="POST" action="php/exemples/parametres.php">
			&lt;label>
				Prénom : &lt;input name="prenom" type="text" value="Jean-Jacques" />
			&lt;/label>
			&lt;label>
				&lt;input name="choix" type="radio" value="creer" checked /> Créer le paramètre de session 'prenom'. 
				&lt;br/>
				&lt;input name="choix" type="radio" value="supprimer" /> Supprimer le paramètre de session 'prenom'.
			&lt;/label>
			&lt;button>Envoyer&lt;/button>
		&lt;/form>
		&lt;br/>
		&lt;?php
			// Traitement du formulaire
			if(isset($_POST["choix"])){
				// Selon le choix de l'utilisateur
				if($_POST["choix"] == "creer"){
					// Création du paramètre de session "prenom"
					$_SESSION["prenom"] = $_POST["prenom"];				
				}else{
					// Suppression du paramètre de session "prenom"
					unset($_SESSION["prenom"]);
				}
			}
			// Affichage du paramètre de session "prenom" s'il est disponible
			if(isset($_SESSION["prenom"])){
				echo "Votre prénom est &lt;b>" . $_SESSION["prenom"] . "&lt;/b> !";
			}
		?>
	</code>
</div>

<div class="resultat">
	<form method="POST" action="php/exemples/parametres.php">
		<label>
			Prénom : <input name="prenom" type="text" value="Jean-Jacques" />
		</label>
		<label>
			<input name="choix" type="radio" value="creer" checked /> Créer le paramètre de session 'prenom'. 
			<br/>
			<input name="choix" type="radio" value="supprimer" /> Supprimer le paramètre de session 'prenom'.
		</label>
		<button>Envoyer</button>
	</form>
	<br/>
	<?php
		// Traitement du formulaire
		if(isset($_POST["choix"])){
			// Selon le choix de l'utilisateur
			if($_POST["choix"] == "creer"){
				// Création du paramètre de session "prenom"
				$_SESSION["prenom"] = $_POST["prenom"];				
			}else{
				// Suppression du paramètre de session "prenom"
				unset($_SESSION["prenom"]);
			}
		}
		// Affichage du paramètre de session "prenom" s'il est disponible
		if(isset($_SESSION["prenom"])){
			echo "Votre prénom est <b>" . $_SESSION["prenom"] . "</b> !";
		}
	?>
</div>

<?php
	
	// Pied de page
	include("../footer.php");
	
?>
