<?php
	
	/**
	 * Quelques fonctions PHP utilisées dans le tutoriel. 
	 * Avertissement: le code de ces fonctions n'a pas vocation à être 
	 * compris par les élèves.
	 **/
	 
	 /* *************************************************** *\
	  *               Gestion des corrections
	 \* *************************************************** */
	
	/**
	 * Teste si l'utiliteur souhaite afficher la correction 
	 * d'un exercice.
	 **/
	function demande_correction(){
		return isset($_GET["correction"]);
	}
	
	/**
	 * Tests si oui ou non les corrections sont disponibles.
	 * @return Boolean
	 **/
	function disponible_correction(){
		return realpath(__DIR__ . "/corrections/.charset") ? false : true;
	}
	
	/**
	 * Retourne la liste des corrigés, la liste des chemins vers les 
	 * fichiers correspondant à des corrections.
	 * @return Array
	 **/
	function liste_correction(){
		// Liste des corrigés (pour le moment vide)
		$liste = array();
		// Ouverture du dossier dans lequel se trouve les corrigés
		$dossier = opendir(realpath(__DIR__ . "/corrections"));
		// Parcours des sous-dossiers du dossier "php/corrections"
		while($exercice = readdir($dossier)){
			// Les dossiers "." et ".." sont des pseudo-dossiers qui permettent 
			// de revenir aux dossiers précédents. Il ne faut pas en tenir compte!
			if($exercice != "." && $exercice != ".."){
				// Chemin courant
				$dossierExercice = realpath(__DIR__ . "/corrections/" . $exercice);
				// On ne se préoccupe que des dossiers
				if(is_dir($dossierExercice)){
					// Ouverture du sous-dossier dans lequel se trouve les corrigés d'un exercice
					$dossierExercice = opendir($dossierExercice);
					// Parcours du sous-dossier "php/corrections/[exercice]"
					while($correction = readdir($dossierExercice)){
						if($correction != "." && $correction != ".."){
							// Ajout du corrigé à la liste des corrigés
							$liste[] = realpath(__DIR__ . "/corrections/" . $exercice . "/" . $correction);
						}
					}
					// On oublie pas de fermer le dossier contenant les corrigés de l'exercice.
					closedir($dossierExercice);
				}
			}
		}
		// On oublie pas de fermer le dossier contenant tous les corrigés.
		closedir($dossier);
		// Finalement, on retourne la liste des corrigés.
		return $liste;
	}
	
	/**
	 * Affiche la correction d'un ou plusieurs exercices si 
	 * celle-ci est disponible. Dans le cas contraire : 
	 * - précise à l'utilisateur que la correction n'est pour le 
	 * moment pas accessible ;
	 * - affiche un formulaire permettant de saisir la clef permettant 
	 * de décrypter les corrigés ;
	 * - traite le formulaire s'il est transmis.
	 **/
	function afficher_correction($fichiers){
		// Traitement du formulaire de décryptage
		if(!disponible_correction()){
			if(isset($_GET["decrypter"], $_POST["clef"])){
				$_POST["clef"] = get_clef($_POST["clef"]);
				$clef = preg_match("/===(.*)===/", $_POST["clef"], $matches);
				if(!isset($matches[1]) || !decrypter_correction($matches[1])){
					$fail = true;
				}
			}
		}
		// Affichage des correction puisqu'elle sont disponibles
		if(disponible_correction()){
			foreach($fichiers as $fichier){
				include("corrections/" . $fichier);
			}
		// Affichage du formulaire de décryptage dans le cas contraire
		}else{
			echo "<div class=\"cryptage\">";
			$action = $_SERVER["REQUEST_URI"];
			if(!isset($_GET["decrypter"])){
				if(strpos($action, "?") === false){
					$action .= "?decrypter";
				}else{
					$action .= "&decrypter";
				}
			}
			echo "
				<big>La correction des exercices n'est pas encore disponible.</big> 
				<br/><br/>
				Afin d'avoir accès au corrigés, veuillez copier/coller ci-dessous 
				la clef permettant de les décrypter, puis cliquer sur le bouton \"décrypter\" :
				<br/><br/>
				<form method=\"POST\" action=\"" . $action . "\">
					<textarea name=\"clef\" placeholder=\"[Copier/coller la clef ici...]\"></textarea>
					<br/><br/>
					<button>Décrypter</button>
				</form>
			";
			if(isset($fail)){
				echo "
				<br/>
				<div class=\"erreur\">
					La clef saisie est incorrecte. Veuillez vous assurer que vous avez bien 
					copié/collé la clef.
				</div>
				";
			}
			echo "</div>";
		}
	}
	
	/**
	 * Traitement
	 **/
	function masquer_correction(){
		// Affichage du formulaire de cryptage
		if(isset($_GET["crypter"])){
			$title = "Cryptage des corrections";
			include("header.php");
			echo "<div class=\"cryptage\">";
			// Affichage du formulaire de cryptage si les corrigés sont disponibles
			if(disponible_correction()){
				echo "
					<big>Les corrections sont actuellement disponibles.</big>
					<br/><br/>
					Afin de masquer les corrigés, veuillez copier/coller ci-dessous une clef 
					de cryptage, puis cliquer sur le bouton \"crypter\" :
					<br/><br/>
					<form method=\"POST\" action=\"" . $_SERVER["REQUEST_URI"] . "\">
						<textarea name=\"clef\" placeholder=\"[Copier/coller la clef ici...]\"></textarea>
						<br/><br/>
						<button>Crypter</button>
						<br/><br/>
						<i>N.B. : la clef doit être délimitée à gauche et à droite par les caractères \"===\".</i>
					</form>
				";
				// Traitement du formulaire de cryptage
				if(isset($_POST["clef"])){
					$_POST["clef"] = get_clef($_POST["clef"]);
					$clef = preg_match("/===(.*)===/", $_POST["clef"], $matches);
					echo "<br/>";
					if(isset($matches[1]) && crypter_correction($matches[1])){
						echo "
							<div class=\"succes\">
								Les corrigés ont été cryptés avec succès.
							</div>
						";
					}else{
						echo "
							<div class=\"erreur\">
								La clef saisie est incorrecte. Veuillez vous assurer que vous avez bien 
								copié/collé la clef.
							</div>
						";
					}
				}
			// Affichage d'un lien vers le formulaire de décryptage dans le cas contraire
			}else{
				echo "
					<big>Les corrections sont déjà indisponibles.</big>
					<br/><br/>
					Afin de décrypter les corrigés et pouvoir les afficher à nouveau, cliquer 
					sur le lien suivant : <a href=\"" . explode("?", $_SERVER["REQUEST_URI"])[0] . "?correction&decrypter\">décrypter</a>
				";
			}
			include("footer.php");
			exit;
		}
	}
	
	/**
	 * Crypte les corrigés des exercices à partir d'un texte 
	 * utilisé comme clef de cryptage.
	 * @param String $clef
	 *		Clef de cryptage à utiliser. La clef de cryptage 
	 *		fournie est délimitée par les caractères.
	 * @return Boolean
	 **/
	function crypter_correction($clef){
		// Liste des corrections
		$corrections = liste_correction();
		// Jeu de caractères
		$charsetFile = __DIR__ . "/corrections/.charset";
		// Pour éviter de crypter les corrections si elles sont déjà cryptées 
		if(!realpath($charsetFile)){
			// Génération et conservation du jeu de caractères
			$charset = charset($corrections, $charsetFile);
			// Génération et conservation de l'empreinte de la clef
			file_put_contents(__DIR__ . "/corrections/.hash", hash("sha256", $clef));
			// Cryptage des corrections une à une
			foreach($corrections as $correction){
				file_put_contents(
					$correction,
					crypter(file_get_contents($correction), $clef, $charset)
				);
			}
			return true;
		}
		return false;
	}
	
	/**
	 * Décrypte les corrigés des exercices à partir de la 
	 * clef de cryptage initiallement utilisée.
	 * @param String $clef
	 *		Clef de cryptage à utiliser.
	 * @return Boolean
	 **/
	function decrypter_correction($clef){
		// Liste des corrections
		$corrections = liste_correction();
		// Jeu de caractères
		$charsetFile = realpath(__DIR__ . "/corrections/.charset");
		// Empreinte de la clef
		$hashFile = realpath(__DIR__ . "/corrections/.hash");
		// Pour éviter de décrypter les corrections si elles sont déjà en clair
		if($charsetFile && $hashFile){
			$charset = file_get_contents($charsetFile);
			$hash = file_get_contents($hashFile);
			// On ne décrypte les corrections que si la bonne clef est fournie
			if($hash == hash("sha256", $clef)){
				// Déryptage des corrections une à une
				foreach($corrections as $correction){
					file_put_contents(
						$correction,
						decrypter(file_get_contents($correction), $clef, $charset)
					);
				}
				unlink($charsetFile);
				unlink($hashFile);
				return true;
			}
		}
		return false;
	}
	
	/* *************************************************** *\
	 *                    Cryptographie
	\* *************************************************** */

	/**
	 * Crypte un texte à partir d'une clef de cryptage et d'un jeu 
	 * de caractères.
	 * @param String $texte 
	 *		Texte à crypter.
	 * @param String $clef 
	 *		Clef de cryptage utilisée.
	 * @param String $charset 
	 *		Jeu de caractères utilisé.
	 * @return String 
	 *		Texte crypté.
	 **/
	function crypter($texte, $clef, $charset){
		$nTexte = strlen($texte);
		$nClef = strlen($clef);
		$nCharset = strlen($charset);
		$assoc = fetch_charset($charset);
		for($i=0; $i<$nTexte; $i++){
			if(isset($assoc[$texte[$i]], $assoc[$clef[$i%$nClef]])){
				$delta = $assoc[$clef[$i%$nClef]];
				$index = $assoc[$texte[$i]];
				$index = ($index + $delta) % $nCharset;
				$texte[$i] = $charset[$index];
			}
		}
		return $texte;
	}
	
	/**
	 * Décrypte un texte à partir de la clef de cryptage et du jeu 
	 * de caractères utilisés initiallement pour crypter le texte.
	 * @param String $texte
	 *		Texte à décrypter.
	 * @param String $clef
	 *		Clef de cryptage utilisée.
	 * @param String $charset
	 *		Jeu de caractères utilisé.
	 * @return String 
	 *		Text décrypté.
	 **/
	function decrypter($texte, $clef, $charset){
		$nTexte = strlen($texte);
		$nClef = strlen($clef);
		$nCharset = strlen($charset);
		$assoc = fetch_charset($charset);
		for($i=0; $i<$nTexte; $i++){
			if(isset($assoc[$texte[$i]], $assoc[$clef[$i%$nClef]])){
				$delta = $assoc[$clef[$i%$nClef]];
				$index = $assoc[$texte[$i]];
				$index = ($index - $delta + $nCharset) % $nCharset;
				$texte[$i] = $charset[$index];
			}
		}
		return $texte;
	}
	
	/**
	 * Nettoie et retourne une clef passer en paramètre.
	 * @param String $clef
	 * @return String
	 **/
	function get_clef($clef){
		return preg_replace("/\s+/", "", $clef);
	}
	
	/**
	 * Génère un jeu de caractères à partir d'une liste de fichier. 
	 * On entend ici par jeu de caracètres la liste (sous forme de chaîne) 
	 * de tous les caractères distincts présents dans les fichiers. 
	 * @param Array 
	 *		Liste de fichiers (chemins).
	 * @param String $destination=null
	 *		Optionnel. Chemin du fichier où exporter le jeu de caractères.
	 * @return String
	 *		Jeux de caractères identifié.
	 **/
	function charset($fichiers, $destination=null){
		$corrections = liste_correction();
		$charset = array();
		foreach($fichiers as $fichier){
			$html = file_get_contents($fichier);
			$html = preg_replace('/[\x00-\x1F-\x7F]/u', '', $html);
			$n = strlen($html);
			for($i=0; $i<$n; $i++){
				$charset[$html[$i]] = true;
			}
		}
		$charset = implode("", array_keys($charset));
		if($destination){
			file_put_contents($destination, $charset);
		}
		return $charset;
	}
	
	/**
	 * Retourne un jeu de caractères, passé sous forme de chaîne, sous 
	 * forme de tableau associatif. Le tableau associe chaque caractère 
	 * à son indice.
	 * @param String $charset
	 *		Jeu de caractères sous forme de chaîne.
	 * @return Array
	 *		Tableau associatif.
	 **/
	function fetch_charset($charset){
		$assoc = array();
		$n = strlen($charset);
		for($i=0; $i<$n; $i++){
			$assoc[$charset[$i]] = $i;
		}
		return $assoc;
	}
	
	masquer_correction();
	
?>

